package cn.rankin.cmsweb.controller.trade;

import cn.rankin.cmsweb.assist.resolver.NeedUser;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.service.trade.order.OrderService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.cms.OrderDetailWebSearchDTO;
import cn.rankin.data.api.cms.OrderSearchWebDTO;
import cn.rankin.data.api.cms.OrderSnapshotWebSearchDTO;
import cn.rankin.data.api.trade.dto.OrderDTO;
import cn.rankin.data.api.trade.entity.OrderProductSnapshot;
import cn.rankin.data.api.trade.vo.OrderDetailVo;
import cn.rankin.data.api.trade.vo.OrderVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;

@RestController
@RequestMapping(value = "/order")
public class OrderController {

    @InitBinder
    protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
        binder.registerCustomEditor(Date.class,
                new CustomDateEditor(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"), true));
    }

    @Autowired
    private OrderService orderService;

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<OrderVo> create(@RequestBody OrderDTO orderDTO) {
        return orderService.create(orderDTO);
    }

    @RequestMapping(value = "/{orderId}", method = RequestMethod.GET)
    public APIResult<OrderVo> getOrder(@PathVariable String orderId) {
        return orderService.getOrder(orderId);
    }

    @RequestMapping(value = {"/list", ""}, method = RequestMethod.GET)
    public APIResult<Page<OrderVo>> search(@NeedUser UserDetails user, OrderSearchWebDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/pay/{orderId}", method = RequestMethod.POST)
    public APIResult<Boolean> pay(@PathVariable("orderId") String orderId) {
        return orderService.pay(orderId);
    }

    @RequestMapping(value = "/{orderId}", method = RequestMethod.DELETE)
    public APIResult<Boolean> cancel(@PathVariable("orderId") String orderId) {
        return orderService.cancel(orderId);
    }

    @RequestMapping(value = "/snapshot", method = RequestMethod.GET)
    public APIResult<Page<OrderProductSnapshot>> search(@NeedUser UserDetails user, OrderSnapshotWebSearchDTO searchDTO) {
        if (!user.isPlatForm()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }

    @RequestMapping(value = "/detail", method = RequestMethod.GET)
    public APIResult<Page<OrderDetailVo>> search(@NeedUser UserDetails user, OrderDetailWebSearchDTO searchDTO) {
        if (user.isMerchant()) {
            searchDTO.setMerchantId(user.getMerchantId());
        }else if (user.isCp()) {
            searchDTO.setCpId(user.getMerchantId());
        }
        return orderService.search(searchDTO);
    }
}
